from cjen import BigTangerine
from cjen.commons import _get_method_params
from cjen.exceptions import _check_instance, _check_meta_exist, _un_support_char
from cjen.mama.meta_data import MetaData

ALL_FIELDS = "ALL_FIELDS"


# TODO 待实现
def validation_all_metas(func):
    """
    使用条件: BigTangerine 及其 子类 对象

    位置：json.factory 或 mysql.factory 之后

    作用：执行对象函数所有 MetaData 参数的中含有asserts装饰器的方法
    :param func:
    :return:
    """

    @_get_method_params(method=func)
    @_check_instance(decorator="orange.assert.validation_fields", expect=BigTangerine)
    def __inner__(ins: BigTangerine, *args, **kwargs):
        pass

    return __inner__


# TODO 增加对装饰器的判断，看是否有 asserts
def validation_meta(*, meta_name: str, fields: str = ALL_FIELDS):
    """
    使用条件: BigTangerine 及其 子类 对象

    位置：json.factory 或 mysql.factory 之后

    作用：指定执行对象函数中 MetaData 参数的中含有asserts装饰器的方法

    :param meta_name: 参数名
    :param fields: 需要执行校验的Field名, 默认是所有Filed, 指定多个以";"分割  例如 "field1;filed2"
    :return:
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_meta_exist(meta_name=meta_name)
        @_un_support_char(content=fields, char="；", err_msg=f"orange.assert.validation_fields Please not use chinese ；")
        @_check_instance(decorator="orange.assert.validation_fields", expect=BigTangerine)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            metas = kwargs.get(meta_name) if type(kwargs.get(meta_name)) == list else [kwargs.get(meta_name)]

            for meta in metas:
                method_names = list(filter(
                    lambda method: callable(getattr(meta, method)) and not method.startswith(
                        "_") and not method.endswith(
                        "_") and method != "factory" and method != "is_class"
                    ,
                    dir(meta)))

                if fields == ALL_FIELDS:  # 执行所有的参数校验
                    for method_name in method_names:
                        getattr(meta, method_name)()
                else:  # 执行部分校验
                    for field in fields.split(";"):
                        if field in method_names:
                            getattr(meta, field)()
                        else:
                            raise Exception(f'{meta_name} do not have {field}')
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def required(func):
    """
    使用条件：MetaData 及其 子类对象
    位置： 不限
    作用：验证某个字段非空
    :param func:
    :return:
    """

    @_check_instance(decorator="operate.assert.required", expect=MetaData)
    def __inner__(self, *args, **kwargs):
        field = func(self, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
        assert self.meta_data.get(field) is not None, f"{field} must not null"
        return field

    return __inner__


def not_equal(*, value):
    """
    使用条件：MetaData 及其 子类对象
    位置： 不限
    作用：验证某个字段不等于某一个值
    :param value:
    :return:
    """

    def __wrapper__(func):
        def __inner__(self, *args, **kwargs):
            field = func(self, *args) if func.__name__ == '__inner__' else func.__name__
            assert self.meta_data.get(field) == value, f"{self.meta_data.get(field)} should equal {value}"
            return field

        return __inner__

    return __wrapper__


def equal(*, value):
    """
    使用条件：MetaData 及其 子类对象
    位置： 不限
    作用：验证某个字段等于某一个值
    :param value:
    :return:
    """

    def __wrapper__(func):
        def __inner__(self, *args, **kwargs):
            field = func(self, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
            assert self.meta_data.get(field) == value, f"{self.meta_data.get(field)} should equal {value}"
            return field

        return __inner__

    return __wrapper__


def in_range(*, ranges: list):
    """
    使用条件：MetaData 及其 子类对象
    位置： 不限
    作用：验证某个字段的值在某个范围内
    :param ranges:
    :return:
    """

    def __wrapper__(func):
        def __inner__(self, *args, **kwargs):
            field = func(self, *args) if func.__name__ == '__inner__' else func.__name__

            assert self.meta_data.get(field) in ranges, f"{self.meta_data.get(field)} is not in  {ranges}"
            return field

        return __inner__

    return __wrapper__


def not_in_range(*, ranges: list):
    """
    使用条件：MetaData 及其 子类对象
    位置： 不限
    作用：验证某个字段的值不在某个范围内
    :param ranges:
    :return:
    """

    def __wrapper__(func):
        def __inner__(self, *args, **kwargs):
            field = func(self, *args) if func.__name__ == '__inner__' else func.__name__
            assert self.meta_data.get(field) not in ranges, f"{self.meta_data.get(field)} is in  {ranges}"
            return field

        return __inner__

    return __wrapper__



import time
from typing import IO

from pymysql.cursors import Cursor

import cjen
# from cjen import BigTangerine, MetaMysql

# TODO 时区转换的装饰器
from cjen.bigtangerine import ContextArgs, BigTangerine
from cjen.commons import _get_method_params
from cjen.exceptions import _check_instance, _check_params_factory
from cjen.mama.meta_data import MetaData, MetaMysql
from cjen.mama.operate.common import value


@cjen.haha(LogPath="", LogName="CJEN.log")
def track_sql(msg: dict, io: IO):
    io.write("{sql}\n".format(**msg))


def type_boolean(*, true, false):
    """
    TODO 待测试
    covert to True or False
    :param true:
    :param false:
    :return:
    """

    def __wrapper__(func):
        def __inner__(ins: MetaData, *args, **kwargs):
            field = func(ins, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
            if ins.meta_data.get(field) == true: return True
            if ins.meta_data.get(field) == false: return False
            raise Exception("type_boolean can not matched")

        return __inner__

    return __wrapper__()


def timezone(*, zone: str): pass


def type_str_datetime(*, fmt: str): pass


def factory(*, cursor: Cursor = None, clazz, sql: str, params=None, size=1, track=False):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象 \n
    位置：放在http.post_mapping 等请求装饰器之后 \n
    作用：创建针对MYSQL 的 MetaData \n
    注意：只支持查询语句

    :param track: 是否打开日志，记录查询SQL
    :param cursor:
    :param size: -1 代表取所有的,
    :param params: sql的查询条件,
    :param sql: 查询的 sql
    :param clazz:
    :return: 支持返回一个对象 或 对象列表
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_params_factory(clazz=MetaMysql)
        @_check_instance(decorator="operate.mysql.factory", expect=BigTangerine)
        def __inner__(ins, *args, **kwargs):

            try:
                mysql_cursor = cursor if cursor else ins.context.get("cursor")
                query_args = ins.context.pick_up(context_args=params) if isinstance(params, ContextArgs) else params
                mysql_cursor.execute(sql, args=query_args)
                if track: track_sql(dict(sql=mysql_cursor.mogrify(sql, args=query_args)))
                values = mysql_cursor.fetchall()
                cols = [col[0] for col in mysql_cursor.description]
                data = [dict(zip(cols, ele)) for ele in values]
                if size != -1:
                    assert mysql_cursor.rowcount == size, f"the record number is not eql {size}"
                for key, val in kwargs.get("method.__annotations__").items():
                    if issubclass(val, clazz) or (
                            "__origin__" in dir(val) and val.__origin__ == list and issubclass(val.__args__[0], clazz)):
                        metas = [MetaData.factory(clazz=clazz, data=ele) for ele in data]
                        for meta in metas:
                            meta.meta_data = meta.meta_source
                            if ins.context:
                                meta.context.update(ins.context)
                        kwargs[key] = metas[0] if size == 1 else metas

                return func(ins, *args, **kwargs)
            except Exception as e:
                raise e
            finally: pass
                # TODO 可能会造成连接池用完
                # mysql_cursor.close()

        return __inner__

    return __wrapper__


def type_datetime_str(*, fmt: str):
    """
    TODO 待测试
    convert datetime to str
    :param fmt:
    :return:
    """

    def __wrapper__(func):
        def __inner__(ins: MetaData, *args, **kwargs):
            field = func(ins, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
            return time.strftime(ins.meta_data.get(field), fmt)

        return __inner__

    return __wrapper__


def type_datetime_stamp(func):
    """
    TODO 待测试
    covert datetime to timestamp
    :param func:
    :return:
    """

    def __inner__(ins: MetaData, *args, **kwargs):
        field = func(ins, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
        time_array = time.strftime(ins.meta_data.get(field), "%Y-%m-%d %H:%M:%S")
        return time.mktime(time_array)

    return __inner__


class TestObj(MetaMysql):
    @value
    def id(self): ...

# cusor = DatabasePool.cursor(host="200.200.101.113", user="root", port=3306, pwd="Admin123",
#                             database="eclinical_design_dev_56")


# class Api(BigTangerine):
#
#     def __init__(self):
#         super().__init__()
#
#     @factory(cursor=cusor, clazz=TestObj, sql="SELECT * FROM eclinical_odm_item WHERE id = 1", size=-1)
#     def xx(self, metas: list[TestObj] = None, **kwargs):
#         for meta in metas:
#             print(meta.id())
#
#     @factory(cursor=cusor, clazz=TestObj,
#              sql="""SELECT * FROM eclinical_odm_item WHERE id = %s and crf_version_id =5 and is_delete = 0""",
#              params=(1,))
#     def yy(self, meta: TestObj = None, **kwargs):
#         print(meta.id())
#
#
# if __name__ == '__main__':
#     Api().yy()
# Api().xx()



import pytest
from multiprocessing import Process

from cjen.tests.mock_service import mock_app


class MockProcess(Process):
    def __init__(self):
        super().__init__()

    def run(self): mock_app.run()


@pytest.fixture(scope='module', autouse=True)
def mock():
    mp = MockProcess()
    mp.start()
    yield
    mp.terminate()



from pyclbr import Function


def __get_annotations__(method):
    if "__closure__" not in dir(method): return None
    if not method.__closure__ and method.__name__ != "__inner__": return method.__annotations__
    for cell in method.__closure__:
        val = __get_annotations__(cell.cell_contents)
        if val: return val
    return None


def _get_method_params(*, method):
    def __wrapper__(func):
        def __inner__(ins, *args, **kwargs):
            # if method.__closure__:
            #     for cell in method.__closure__:
            #         if "__closure__" not in dir(cell.cell_contents) and not cell.cell_contents.__closure__:
            #             kwargs["method.__annotations__"] = cell.cell_contents.__annotations__
            # else:
            #     kwargs["method.__annotations__"] = method.__annotations__
            kwargs["method.__annotations__"] = __get_annotations__(method)
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__



import json
import os.path

from flask import Flask
from flask import request, Response, send_file

mock_app = Flask(__name__)
__mock_data = {"companies": [dict(id=1, name="C01"), dict(id=2, name="C02")],
               "employees": [dict(id=1, name="E01", company_id=1),
                             dict(id=2, name="E02", company_id=1),
                             dict(id=3, name="E03", company_id=1),
                             dict(id=4, name="E04", company_id=2),
                             dict(id=5, name="E05", company_id=2),
                             ]}


def token_in_body(token: str):
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200,
                           "payload": {"jwtAuthenticationResponse": {"token": token},
                                       "message": "", "exception": None}})
    return rsp


def token_in_header(token: str):
    rsp = base_json_response()
    rsp.headers["token"] = token
    return rsp


def base_json_response():
    rsp = Response()
    rsp.status_code = 200
    rsp.headers = {"Content-Type": "application/json"}
    return rsp


@mock_app.route("/init_in_header", methods=["GET"])
def init_in_header(): return token_in_header("Jwt Init Token In Header")


@mock_app.route("/init_in_body", methods=["GET"])
def init_in_body(): return token_in_body("Jwt Init Token In Body")


@mock_app.route("/exchange_in_body", methods=["GET"])
def exchange_in_body(): return token_in_body("Jwt Exchange Token In Body")


@mock_app.route("/exchange_in_header", methods=["GET"])
def exchange_in_header(): return token_in_header("Jwt Exchange Token In Header")


@mock_app.route("/refresh_in_header", methods=["GET"])
def refresh_in_header(): return token_in_header("Jwt Refresh Token In Header")


@mock_app.route("/refresh_in_body", methods=["GET"])
def refresh_in_body(): return token_in_body("Jwt Refresh Token In Body")


@mock_app.route("/post_method_json", methods=["POST"])
def post_method_json():
    if request.json["username"] and request.json["pwd"]:
        rsp = base_json_response()
        rsp.data = json.dumps({"procCode": 200})
        return rsp


@mock_app.route("/post_method_variable/<int:id>", methods=["POST"])
def post_method_variable(id):
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "path_variable": id})
    return rsp


@mock_app.route("/post_method_path_variable", methods=["POST"])
def post_method_path_variable():
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "path_variable": int(request.args.get("id"))})
    return rsp


@mock_app.route("/put_method_json", methods=["PUT"])
def put_method_json():
    if request.json["username"] and request.json["pwd"]:
        rsp = base_json_response()
        rsp.data = json.dumps({"procCode": 200})
        return rsp


@mock_app.route("/put_method_variable/<int:id>", methods=["PUT"])
def put_method_variable(id):
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "path_variable": id})
    return rsp


@mock_app.route("/put_method_path_variable", methods=["PUT"])
def put_method_path_variable():
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "path_variable": int(request.args.get("id"))})
    return rsp


@mock_app.route("/delete_method_json", methods=["DELETE"])
def delete_method_json():
    if request.json["username"] and request.json["pwd"]:
        rsp = base_json_response()
        rsp.data = json.dumps({"procCode": 200})
        return rsp


@mock_app.route("/delete_method_variable/<int:id>", methods=["DELETE"])
def delete_method_variable(id):
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "path_variable": id})
    return rsp


@mock_app.route("/delete_method_path_variable", methods=["DELETE"])
def delete_method_path_variable():
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "path_variable": int(request.args.get("id"))})
    return rsp


@mock_app.route("/get_method_variable/<int:id>", methods=["GET"])
def get_method_variable(id):
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "path_variable": id})
    return rsp


@mock_app.route("/get_method_path_variable", methods=["GET"])
def get_method_path_variable():
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "path_variable": int(request.args.get("id"))})
    return rsp


@mock_app.route("/download_file", methods=["POST"])
def download_file():
    return send_file('download_file.txt')


@mock_app.route("/upload_file", methods=["POST"])
def upload_file():
    file = request.files['file']
    file.save(os.path.join(os.path.dirname(__file__), "aiai", "upload_target.txt"))
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200})
    return rsp


@mock_app.route("/test_headers", methods=["GET"])
def headers_tester():
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "headers": dict(request.headers)})
    return rsp


@mock_app.route("/company/<int:company_id>", methods=["GET", "POST", "PUT", "DELETE"])
def company(company_id):
    rsp = base_json_response()
    ret = list(filter(lambda c: c.get("id") == company_id, __mock_data["companies"]))[0]
    rsp.data = json.dumps({**{"procCode": 200}, **ret})
    return rsp


@mock_app.route("/companies", methods=["GET"])
def all_companies():
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "companies": __mock_data["companies"]})
    return rsp


@mock_app.route("/all_employees", methods=["GET"])
def all_employees():
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "employees": __mock_data["employees"]})
    return rsp


@mock_app.route("/none_field_response", methods=["GET"])
def none_field_response():
    rsp = base_json_response()
    rsp.data = json.dumps({"procCode": 200, "employees": None})
    return rsp


if __name__ == '__main__':
    mock_app.run()



import pytest

from cjen.bigtangerine import ContextManager, ContextArgs


@pytest.fixture(scope='function')
def context_data():
    context = ContextManager()
    data = dict(x=1, y=2)
    context.update(data)
    return context


def test_update_dict(context_data):
    data = dict(x=3, y=4)
    context_data.update(data)
    assert context_data.get("x") == 3
    assert context_data.get("y") == 4
    assert context_data["x"] == 3
    assert context_data["y"] == 4


def test_update_tuple(context_data):
    context_data.update(z=1, j=2)
    assert context_data.get("z") == 1
    assert context_data["z"] == 1
    assert context_data.get("j") == 2
    assert context_data["j"] == 2


def test_update_context(context_data):
    target = ContextManager()
    target.update(context_data)
    assert target["x"] == 1
    assert target["y"] == 2


def test_pick_up(context_data):
    pick = context_data.pick_up(context_args=ContextArgs(c="x", d="y"))
    assert pick["c"] == 1
    assert pick["d"] == 2



import os.path
import cjen
from cjen import BigTangerine


class MockService(BigTangerine):
    @cjen.http.base_url(uri="http://127.0.0.1:5000")
    def __init__(self): super().__init__()

    @cjen.headers.contentType(value="application/json")
    @cjen.http.post_mapping(uri="post_method_json")
    def post_method_json(self, *, data, resp=None, **kwargs):
        assert resp.get("procCode") == 200

    @cjen.http.post_mapping(uri="post_method_variable/{id}")
    def post_method_variable(self, *, path_variable: dict, resp=None, **kwargs):
        assert resp.get("procCode") == 200
        assert resp.get("path_variable") == path_variable.get("id")

    @cjen.http.post_mapping(uri="post_method_path_variable?id={id}")
    def post_method_path_variable(self, *, path_variable: dict, resp=None, **kwargs):
        assert resp.get("procCode") == 200
        assert resp.get("path_variable") == path_variable.get("id")

    @cjen.http.post_mapping(uri="download_file")
    def download_file(self, *, resp=None, **kwargs):
        path = os.path.join(os.path.dirname(__file__), "test_download_file")
        with open(path, "wb") as f:
            f.write(resp)
        with open(path, "r") as f:
            assert f.read() == "TestDownLoadFile"
        os.remove(path)

    @cjen.http.get_mapping(uri="get_method_path_variable?id={id}")
    def get_method_path_variable(self, *, path_variable: dict, resp=None, **kwargs):
        assert resp.get("procCode") == 200
        assert resp.get("path_variable") == path_variable.get("id")

    @cjen.http.get_mapping(uri="get_method_variable/{id}")
    def get_method_variable(self, *, path_variable: dict, resp=None, **kwargs):
        assert resp.get("procCode") == 200
        assert resp.get("path_variable") == path_variable.get("id")

    @cjen.headers.contentType(value="application/json")
    @cjen.http.put_mapping(uri="put_method_json")
    def put_method_json(self, *, data, resp=None, **kwargs): ...

    @cjen.http.put_mapping(uri="put_method_variable/{id}")
    def put_method_variable(self, *, path_variable, resp=None, **kwargs): ...

    @cjen.http.put_mapping(uri="put_method_path_variable?id={id}")
    def put_method_path_variable(self, *, path_variable, resp=None, **kwargs): ...

    @cjen.headers.contentType(value="application/json")
    @cjen.http.delete_mapping(uri="delete_method_json")
    def delete_method_json(self, *, data, resp=None, **kwargs): ...

    @cjen.http.delete_mapping(uri="delete_method_variable/{id}")
    def delete_method_variable(self, *, path_variable: dict, resp=None, **kwargs): ...

    @cjen.http.delete_mapping(uri="delete_method_path_variable?id={id}")
    def delete_method_path_variable(self, *, path_variable: dict, resp=None, **kwargs): ...

    @cjen.http.upload_mapping(uri="upload_file")
    def upload_file(self, *, data, resp=None, **kwargs):
        assert resp.get("procCode") == 200
        path = os.path.join(os.path.dirname(__file__), "upload_target.txt")
        with open(path, "r") as f:
            assert f.read() == "TestUploadFile"
        os.remove(path)


def test_post():
    mock = MockService()
    mock.post_method_json(data=dict(username="xx", pwd="yyy"))
    mock.post_method_variable(path_variable=dict(id=1))
    mock.post_method_path_variable(path_variable=dict(id=1))


def test_download():
    mock = MockService()
    mock.download_file()


def test_get():
    mock = MockService()
    mock.get_method_variable(path_variable=dict(id=2))
    mock.get_method_path_variable(path_variable=dict(id=2))


def test_upload():
    mock = MockService()
    mock.upload_file(data=dict(file=os.path.join(os.path.dirname(__file__), "upload_source.txt")))


def test_put():
    mock = MockService()
    mock.put_method_json(data=dict(username="xx", pwd="yyy"))
    mock.put_method_variable(path_variable=dict(id=1))
    mock.put_method_path_variable(path_variable=dict(id=1))


def test_delete():
    mock = MockService()
    mock.delete_method_json(data=dict(username="xx", pwd="yyy"))
    mock.delete_method_variable(path_variable=dict(id=1))
    mock.delete_method_path_variable(path_variable=dict(id=1))



from cjen.mama.meta_data import MetaData


def value(func):
    """
    使用条件：MetaData 或 其子类使用

    使用场景：如果期望返回 MetaData中定义的 Field的值，则需使用该装饰器

    位置： 装饰函数的顶层装饰器

    作用: 返回 MetaData 中 属性的值

    :param func:
    :return: 返回字段值
    """

    def __inner__(ins: MetaData, *args, **kwargs):
        field = func(ins, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
        return ins.meta_data.get(field)

    return __inner__


# TODO 待实现
def change_key_in_dict(*, source: list[str], change_to: list[str]):
    def __wrapper__(func):
        def __inner__(ins: MetaData, *args, **kwargs):
            pass

        return __inner__

    return __wrapper__


# TODO 待实现
def change_key_in_list(*, source: list[str], change_to: list[str]):
    def __wrapper__(func):
        def __inner__(ins: MetaData, *args, **kwargs):
            pass

        return __inner__

    return __wrapper__



import os
from typing import IO

import cjen


@cjen.haha(LogPath=os.path.curdir, LogName="test_delay.log", Mode='a')
def logs(msg: dict, io: IO):
    io.write("{number}\n".format(**msg))


def test_delay():
    for i in range(10):
        logs(dict(number=i))



import os
import threading
import time
from typing import IO

import cjen
from cjen.dada.smile import SmileVoice

test_data = {"single.log": [list(range(1, 20)),
                            list(range(21, 40)),
                            list(range(41, 60)),
                            list(range(61, 80)),
                            list(range(81, 100))]}


def setup_function():
    for log_file in list(test_data.keys()):
        if os.path.exists(log_file): os.remove(log_file)


@cjen.haha(LogPath=os.path.curdir, LogName=list(test_data.keys())[0], Mode='a')
def logs1(msg: dict, io: IO):
    io.write("{word}\n".format(**msg))


@cjen.haha(LogPath=os.path.curdir, LogName=list(test_data.keys())[0], Mode='a')
def logs2(msg: dict, io: IO):
    io.write("{word}\n".format(**msg))


@cjen.haha(LogPath=os.path.curdir, LogName=list(test_data.keys())[0], Mode='a')
def logs3(msg: dict, io: IO):
    io.write("{word}\n".format(**msg))


@cjen.haha(LogPath=os.path.curdir, LogName=list(test_data.keys())[0], Mode='a')
def logs4(msg: dict, io: IO):
    io.write("{word}\n".format(**msg))


@cjen.haha(LogPath=os.path.curdir, LogName=list(test_data.keys())[0], Mode='a')
def logs5(msg: dict, io: IO):
    io.write("{word}\n".format(**msg))


def thread_log(log, values: list[int]):
    for _ in values: log(dict(word='a'))


def test_smile_parallel2():
    """
    测试 多线程写入相同的文件
    :return:
    """
    for i, log in enumerate([logs1, logs2, logs3, logs4, logs5]):
        t = threading.Thread(target=thread_log, args=(log, list(test_data.values())[0][i]))
        t.start()
        t.join()
    while len(list(filter(lambda thd: thd.name == SmileVoice.ThreadName and not thd.queue.empty(),
                          threading.enumerate()))) != 0:
        time.sleep(1)

    for log in list(test_data.keys()):
        with open(os.path.join(os.path.curdir, log), 'r') as f:
            assert sum([len(array) for array in list(test_data.values())[0]]) == len(f.readlines())



import jsonpath

from cjen import BigTangerine
from enum import Enum

from cjen.commons import _get_method_params
from cjen.exceptions import JwtWrongErr


class JWTFrom(Enum):
    BODY = 0,
    HEADER = 1


class JWTAction(Enum):
    INIT = 101
    EXCHANGE = 102
    REFRESH = 103


def jwt(*, key: str, json_path: str, jwt_from: JWTFrom = JWTFrom.BODY, action: JWTAction = JWTAction.INIT):
    """
    主要用于JWT登陆方式:1. 在Header中初始化 jwt token,2. 在Header中交换 jwt token

    目前只支持返回格式为json

    :param action: INIT: 登陆时获取token, EXCHANGE: 中间过程交换token, REFRESH:接口交互中实时刷新token
    :param jwt_from: 用于交换的token的位置 JWTFrom.BODY 响应体中, JWTFrom.HEADER 响应头中
    :param key: header中 存放token的key
    :param json_path: 获取token 的json 路径
    :return:
    """

    def __wrapper__(func):
        # @_get_method_params(method=func)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            rsp = kwargs.get("resp") if jwt_from == JWTFrom.BODY else dict(kwargs.get("response_content").headers)
            if not jsonpath.jsonpath(rsp, json_path):
                if action == JWTAction.INIT or action == JWTAction.EXCHANGE:
                    raise JwtWrongErr(f"can not find jwt key in {rsp}")
            else:
                ins.headers[key] = jsonpath.jsonpath(rsp, json_path)[0] if json_path else rsp
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__



import os
from enum import unique, Enum

import requests
import yaml
from requests_toolbelt import MultipartEncoder

from cjen.mama.operate.json import factory
from cjen.bigtangerine import BigTangerine
from cjen.commons import _get_method_params
from cjen.exceptions import _check_uri, NetWorkErr, _check_instance


def _multipart_form(func):
    def __inner__(ins: BigTangerine, *args, **kwargs):
        data_form = kwargs.get("data")
        files = {}
        others = {}
        for k, v in data_form.items():
            if os.path.exists(v) and os.path.isfile(v):
                files[k] = v
            else:
                others[k] = v
        if not files: raise Exception("there is no file!")
        for k, v in files.items():
            with open(v, 'rb') as f:
                *rest, postfix = f.name.split(".")
                files[k] = (os.path.basename(f.name), f.read(), _file_type(file_postfix=f".{postfix}"))

        kwargs["form_data"] = MultipartEncoder(fields={**files, **others})
        ins.headers["Content-Type"] = kwargs["form_data"].content_type
        return func(ins, *args, **kwargs)

    return __inner__


def _file_type(*, file_postfix):
    with open(os.path.join(os.path.dirname(__file__), "file-content-type.yaml"), "r") as f:
        content_type = yaml.load(f.read(), Loader=yaml.FullLoader).get("ContentType")
        if content_type.get(file_postfix):
            return content_type.get(file_postfix)
    raise Exception(f"{file_postfix} can not support !")


@unique
class ContentType(Enum):
    JSON = "json"
    FILE_STREAM = "stream"


def _response(func):
    def __inner__(ins: BigTangerine, *args, **kwargs):

        if kwargs.get("resp").status_code == 200:
            if "application/json" in kwargs.get("resp").headers.get("Content-Type"):
                kwargs["response_content"] = kwargs.get("resp")
                kwargs["resp"] = kwargs.get("resp").json() if kwargs.get("resp").content else None
                return func(ins, *args, **kwargs)
            kwargs["resp"] = kwargs.get("resp").content
            return func(ins, *args, **kwargs)
        else:
            raise NetWorkErr(f'{kwargs.get("url")} {kwargs.get("resp").status_code}')

    return __inner__


def _url(*, uri: str):
    def __wrapper__(func):
        def __inner__(ins: BigTangerine, *args, **kwargs):
            path_variable = kwargs.get("path_variable")
            kwargs["url"] = "/".join([ins.base_url, uri])
            if path_variable:
                kwargs["url"] = kwargs["url"].format(**path_variable)
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def base_url(*, uri: str):
    def __wrapper__(func):
        def __inner__(ins: BigTangerine, *args, **kwargs):
            ins.__setattr__("base_url", uri)
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _delete(func):
    @_http_headers
    @_http_json(method=requests.delete)
    @_http_default(method=requests.delete)
    def __inner__(ins: BigTangerine, *args, **kwargs):
        return func(ins, *args, **kwargs)

    return __inner__


def _http_headers(func):
    def __inner__(ins: BigTangerine, *args, **kwargs):
        kwargs["headers"] = {**ins.headers, **kwargs.get("headers")} if kwargs.get("headers") else ins.headers
        return func(ins, *args, **kwargs)

    return __inner__


def _resp_is_not_none(**kwargs): return kwargs.get("resp") is None


def _have_content_type(**kwargs): return kwargs["headers"].get("Content-Type") is not None


def _is_json(**kwargs): return "json" in kwargs["headers"].get("Content-Type")


def _is_upload(**kwargs): return "multipart/form-data" in kwargs["headers"].get("Content-Type")


def _http_json(*, method):
    def __wrapper__(func):
        def __inner__(ins: BigTangerine, *args, **kwargs):
            if _resp_is_not_none(**kwargs) and _have_content_type(**kwargs) and _is_json(**kwargs):
                kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], json=kwargs.get("data"))
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _http_file(*, method):
    def __wrapper__(func):
        def __inner__(ins: BigTangerine, *args, **kwargs):
            if _resp_is_not_none(**kwargs) and _have_content_type(**kwargs) and _is_upload(**kwargs):
                kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], data=kwargs.get("form_data"))
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _http_default(*, method):
    def __wrapper__(func):
        def __inner__(ins: BigTangerine, *args, **kwargs):
            if _resp_is_not_none(**kwargs):
                kwargs["resp"] = method(url=kwargs["url"], headers=kwargs["headers"], data=kwargs.get("data"))
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _get(func):
    def __inner__(ins: BigTangerine, *args, **kwargs):
        headers = {**ins.headers, **kwargs.get("headers")} if kwargs.get("headers") else ins.headers
        kwargs["resp"] = requests.get(url=kwargs["url"], headers=headers, params=kwargs.get("params"))
        return func(ins, *args, **kwargs)

    return __inner__


def _post(func):
    @_http_headers
    @_http_json(method=requests.post)
    @_http_file(method=requests.post)
    @_http_default(method=requests.post)
    def __inner__(ins: BigTangerine, *args, **kwargs):
        return func(ins, *args, **kwargs)

    return __inner__


def _put(func):
    @_http_headers
    @_http_json(method=requests.put)
    @_http_default(method=requests.put)
    def __inner__(ins: BigTangerine, *args, **kwargs):
        return func(ins, *args, **kwargs)

    return __inner__


def get_mapping(*, uri: str, json_clazz=None):
    """
    使用范围：BigTangerine 或其 子类对象

    位置：装饰函数的上层装饰器. 如果有使用Header装饰器，在Header装饰器之后，如果没有 则是顶层装饰器

    发送 GET 请求，并返回结果

    :param json_clazz: 如果期望返回数据直接生成MetaJson, 则指定该生成的类型
    :param uri:
    :return:
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="http.get_mapping", expect=BigTangerine)
        @_check_uri(uri=uri)
        @_url(uri=uri)
        @_get
        @_response
        @factory(clazz=json_clazz)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            func(ins, *args, **kwargs)
            return kwargs.get("resp")

        return __inner__

    return __wrapper__


def post_mapping(*, uri: str, json_clazz=None):
    """
    使用范围：BigTangerine 或其 子类对象

    位置：装饰函数的上层装饰器，如果有使用Header装饰器，在Header装饰器之后，如果没有 则是顶层装饰器

    发送 Post 请求，并返回结果

    :param json_clazz:
    :param uri:
    :return:
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="http.post_mapping", expect=BigTangerine)
        @_check_uri(uri=uri)
        @_url(uri=uri)
        @_post
        @_response
        @factory(clazz=json_clazz)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            func(ins, *args, **kwargs)
            return kwargs.get("resp")

        return __inner__

    return __wrapper__


def put_mapping(*, uri: str, json_clazz=None):
    """
    使用范围：BigTangerine 或其 子类对象

    位置：装饰函数的上层装饰器，如果有使用Header装饰器，在Header装饰器之后，如果没有 则是顶层装饰器

    发送 PUT 请求，并返回结果

    :param json_clazz:
    :param uri:
    :return:
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="http.put_mapping", expect=BigTangerine)
        @_check_uri(uri=uri)
        @_url(uri=uri)
        @_put
        @_response
        @factory(clazz=json_clazz)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            func(ins, *args, **kwargs)
            return kwargs.get("resp")

        return __inner__

    return __wrapper__


def upload_mapping(*, uri: str, json_clazz=None):
    """
    使用范围：BigTangerine 或其 子类对象

    位置：装饰函数的上层装饰器，如果有使用Header装饰器，在Header装饰器之后，如果没有 则是顶层装饰器

    发送 上传文件 请求，并返回结果

    :param json_clazz:
    :param uri:
    :return:
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="http.upload_mapping", expect=BigTangerine)
        @_check_uri(uri=uri)
        @_url(uri=uri)
        @_multipart_form
        @_post
        @_response
        @factory(clazz=json_clazz)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            func(ins, *args, **kwargs)
            return kwargs.get("resp")

        return __inner__

    return __wrapper__


def delete_mapping(*, uri: str, json_clazz=None):
    """
    使用范围：BigTangerine 或其 子类对象

    位置：位置：装饰函数的上层装饰器，如果有使用Header装饰器，在Header装饰器之后，如果没有 则是顶层装饰器

    发送 delete 请求，并返回结果

    :param json_clazz:
    :param uri:
    :return:
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="http.delete_mapping", expect=BigTangerine)
        @_check_uri(uri=uri)
        @_url(uri=uri)
        @_delete
        @_response
        @factory(clazz=json_clazz)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            func(ins, *args, **kwargs)
            return kwargs.get("resp")

        return __inner__

    return __wrapper__



import pytest

from cjen import BigTangerine
import cjen


class HeaderMockService(BigTangerine):

    @cjen.http.base_url(uri="http://127.0.0.1:5000")
    @cjen.headers.basicHeaders(headers=dict(Basicheader="1"))
    def __init__(self): super().__init__()

    @cjen.http.get_mapping(uri="test_headers")
    def basic_headers(self, resp=None, **kwargs):
        assert resp.get("headers")["Basicheader"] == self.headers["Basicheader"]

    @cjen.headers.appendBasicHeaders(headers=dict(Appendbasicheaders="2"))
    @cjen.http.get_mapping(uri="test_headers")
    def append_basic_headers(self, resp=None, **kwargs):
        assert resp.get("headers")["Basicheader"] == self.headers["Basicheader"]
        assert resp.get("headers")["Appendbasicheaders"] == self.headers["Appendbasicheaders"]

    @cjen.headers.addHeaders(headers=dict(Newheader="3"))
    @cjen.http.get_mapping(uri="test_headers")
    def add_headers(self, resp=None, **kwargs):
        assert resp.get("headers")["Basicheader"] == self.headers["Basicheader"]
        assert resp.get("headers")["Newheader"] == "3"
        assert self.headers.get("Newheader") is None

    @cjen.headers.contentType(value="4")
    @cjen.http.get_mapping(uri="test_headers")
    def add_content_type(self, resp=None, **kwargs):
        assert resp.get("headers")["Basicheader"] == self.headers["Basicheader"]
        assert resp.get("headers")["Content-Type"] == "4"
        assert self.headers.get("Content-Type") is None

    @cjen.headers.accept(value="5")
    @cjen.http.get_mapping(uri="test_headers")
    def add_accept(self, resp=None, **kwargs):
        assert resp.get("headers")["Basicheader"] == self.headers["Basicheader"]
        assert resp.get("headers")["Accept"] == "5"
        assert self.headers.get("Accept") is None


def test_basic_headers():
    mock = HeaderMockService()
    mock.basic_headers()


def test_append_basic_headers():
    mock = HeaderMockService()
    mock.append_basic_headers()


def test_add_headers():
    mock = HeaderMockService()
    mock.add_headers()


def test_add_content_type():
    mock = HeaderMockService()
    mock.add_content_type()


def test_add_accept():
    mock = HeaderMockService()
    mock.add_accept()



class ContextArgs(dict):
    def __init__(self, seq=None, **kwargs):
        if seq:
            super(ContextArgs, self).__init__(seq, **kwargs)
        else:
            super(ContextArgs, self).__init__(**kwargs)


class ContextManager(object):
    def __init__(self):
        self.__cache = {}

    def update(self, E=None, **F):
        if E is not None and isinstance(E, ContextManager):
            self.__cache.update(E.__cache, **F)
        elif E is not None:
            self.__cache.update(E, **F)
        else:
            self.__cache.update(**F)

    def pick_up(self, *, context_args: ContextArgs):
        return dict(zip(list(context_args.keys()), [self.get(key) for key in list(context_args.values())]))

    @property
    def content(self):
        return self.__cache

    def pop(self, key):
        return self.__cache.pop(key)

    def get(self, key):
        return self.__cache.get(key)

    def __getitem__(self, key):
        return self.__cache.__getitem__(key)

    def __setitem__(self, key, value):
        self.__cache.__setitem__(key, value)


class BigTangerine(object):
    """
    主要的工作类，运行的对象需继承改类型，才能使用一系列装饰器

    定义的方法主要可以如下几种例子:

        针对 POST 请求:

        def method_name(self, data, resp=None, meta_json:MetaJson=None, meta_mysql:MetaMysql=None, **kwargs)

        针对 GET 请求:

        def method_name(self, params, resp=None, meta_json:MetaJson=None, meta_mysql:MetaMysql=None, **kwargs)

        针对 PUT:

        def method_name(self, data, resp=None, meta_json:MetaJson=None, meta_mysql:MetaMysql=None, **kwargs)

        针对 url 中带有参数的请求,以 POST 请求为例:

        def method_name(self, path_variable:dict, data, resp=None, meta_json:MetaJson=None, meta_mysql:MetaMysql=None, **kwargs)
    参数的解释

        data, params, path_variable 为请求参数，根据请求类型固定名称

        resp 为接口返回内容，参数名为固定

        meta_json: 可选参数 需搭配@orange.json.factory 使用

        meta_mysql: 可选参数 需搭配@orange.mysql.factory 使用

        ** kwarg: 固定写法

    """

    def __init__(self): ...

    def __new__(cls, *args, **kwargs):
        self = super(BigTangerine, cls).__new__(cls)
        self.headers = {}
        self.context = ContextManager()
        cls.check_methods(self)
        return self

    # TODO 增加BigTangerine 中定义方法的检查
    @classmethod
    def check_methods(cls, ins): pass

    @classmethod
    def is_class(cls, instance):
        return isinstance(instance, BigTangerine)



import os


class FileHelper(object):

    @classmethod
    def read(cls, *path):
        with open(os.path.join(*path), 'r') as f:
            return f.read()

    @classmethod
    def cur_read(cls, *, cur, file):
        """
        与期望文件文件同一路径文件
        :param cur: 文件的__file__参数
        :param file: 文件名
        :return:
        """
        return cls.read(os.path.dirname(cur), file)

    @classmethod
    def replace(cls, rep: dict, *path):
        with open(os.path.join(*path), 'r') as f:
            return f.read().format(**rep)

    @classmethod
    def readlines_str(cls, *path):
        with open(os.path.join(*path), 'r') as f:
            return [line.rstrip() for line in f.readlines()]

    @classmethod
    def readlines_int(cls, path: str):
        with open(path, 'r') as f:
            return [int(line.rstrip()) for line in f.readlines()]



import jsonpath

from cjen import BigTangerine
from cjen.commons import _get_method_params
from cjen.exceptions import JsonPathNotFoundErr, _check_instance, _check_params_factory
from cjen.mama.meta_data import MetaData, MetaJson


def factory(*, clazz):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象

    位置：放在http.post_mapping 等请求装饰器之后

    作用: 生成 MetaJson 对象

    注意：1. 只支持 obj:Class ，不支持 list[Class] 类型

    :param clazz: MetaJson 或其 子类
    :return: 返回单个对象
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_params_factory(clazz=MetaJson)
        @_check_instance(decorator="operate.json.factory", expect=BigTangerine)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            for key, value in kwargs.get("method.__annotations__").items():
                if issubclass(value, MetaJson):
                    kwargs[key] = MetaData.factory(clazz=clazz, data=kwargs.get("resp"))
                    # 同步上下文
                    # 把BigTangerine 中的 上下文同步到 MetaData中的上下文
                    if ins.context:
                        kwargs[key].context.update(ins.context)
            return func(ins, *args, **kwargs)

        def __empty__(ins: BigTangerine, *args, **kwargs):
            return func(ins, *args, **kwargs)

        if clazz is None: return __empty__
        return __inner__

    return __wrapper__


def one(*, json_path: str):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象

    位置：紧接被装饰函数，与 many 可调换顺序

    作用: 从Json对象中

    使用注意：
        1. 只支持 obj:Class ，不支持 list[Class] 类型
    :return:
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="operate.json.one", expect=MetaData)
        def __inner__(ins: MetaData, *args, **kwargs):
            field = func(ins, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
            json_data = jsonpath.jsonpath(ins.meta_source, json_path.format(**ins.context.content))
            if not json_data:
                raise JsonPathNotFoundErr(f"can not get the data from {json_path.format(**ins.context.content)}")
            ins.meta_data[field] = json_data[0]
            return field

        return __inner__

    return __wrapper__


def listOf(*, clazz):
    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="operate.json.many", expect=MetaData)
        def __inner__(ins: MetaData, *args, **kwargs):
            field = func(ins, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
            ins.meta_data[field] = [MetaJson.factory(clazz=clazz, data=data) for data in ins.meta_data.get(field)]
            return field

        return __inner__

    return __wrapper__


def many(*, json_path: str, filter_keys: list[str] = None):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象

    位置：紧接被装饰函数，与 one 可调换顺序

    适用场景：在 json文件中选择匹配某个属性 {
        user:[
            {name="甲", type="1"},

            {name="乙", type="1"},

            {name="丙", type="2"},

        ]

    }
    的所有值,选取符合"$.user[?＠type==1]"的所有值
    作用: 从Json对象中获取满足条件的值

    :param json_path:
    :param filter_keys: 获取指定的Key-Value，如果不设置的则获取所有的Key-Value
    :return: 返回为一个list对象
    """

    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="operate.json.many", expect=MetaData)
        def __inner__(ins: MetaData, *args, **kwargs):
            field = func(ins, *args, **kwargs) if func.__name__ == '__inner__' else func.__name__
            json_data = jsonpath.jsonpath(ins.meta_source, json_path)
            if not json_data: raise JsonPathNotFoundErr(f"can not get the data from {json_path}")
            ins.meta_data[field] = json_data if not filter_keys else [dict(
                zip(filter_keys, [obj.get(key) for key in filter_keys])) for obj in json_data]
            return field

        return __inner__

    return __wrapper__



from cjen.bigtangerine import BigTangerine
from cjen.aiai.jwt import JWTAction
from cjen.aiai.jwt import JWTFrom
from cjen.aiai.jwt import jwt
from cjen.dada.smile import haha
from cjen.aiai import http
from cjen.aiai import headers
from cjen import context
from cjen.mama.meta_data import MetaJson
from cjen.mama.meta_data import MetaMysql
from cjen.mama import operate
from cjen.nene.database_pool import DatabasePool
__all__ = [
    "haha",
    "BigTangerine",
    "http",
    "headers",
    "jwt",
    "JWTFrom",
    "JWTAction",
    "operate",
    "MetaJson",
    "MetaMysql",
    "DatabasePool"
]



from cjen import BigTangerine
from cjen.exceptions import _check_instance


def add(*, content: dict):
    def __wrapper__(func):
        @_check_instance(decorator="context.add", expect=BigTangerine)
        def __inner__(ins: BigTangerine, *args, **kwargs):
            ins.context.update(content)
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def remove(*, key: str):
    @_check_instance(decorator="context.remove", expect=BigTangerine)
    def __wrapper__(func):
        def __inner__(ins: BigTangerine, *args, **kwargs):
            ins.context.pop(key)
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__







from cjen import BigTangerine
from cjen.commons import _get_method_params
from cjen.exceptions import _check_method, _check_instance


def basicHeaders(*, headers: dict):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象

    位置：该装饰器一般放在 __init__ 函数中

    作用: 初始化Basic Header, Basic Header 是作为基准 Header 存在

    :param headers:
    :return:
    """
    def __wrapper__(func):
        # @_get_method_params(method=func)
        @_check_instance(decorator="headers.basicHeaders", expect=BigTangerine)
        @_check_method(decorator="headers.basicHeaders", method=func, expect="__init__")
        def __inner__(instance: BigTangerine, *args, **kwargs):
            instance.headers.update(headers)
            func(instance, *args, **kwargs)

        return __inner__

    return __wrapper__


def appendBasicHeaders(*, headers: dict):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象

    位置：Header 的 装饰器一般都放在顶层，在其它装饰器之前

    作用: 增加Basic Header

    :param headers:
    :return:
    """
    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="headers.appendBasicHeaders", expect=BigTangerine)
        def __inner__(instance: BigTangerine, *args, **kwargs):
            instance.headers.update(headers)
            return func(instance, *args, **kwargs)

        return __inner__

    return __wrapper__


def accept(*, value: str):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象

    位置：针对装饰器所在的函数，发送请求时，临时新增的 Accept Header.

    作用: 新增临时 Accept Header

    :param value:
    :return:
    """
    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="headers.accept", expect=BigTangerine)
        def __inner__(instance: BigTangerine, *args, **kwargs):
            kwargs["headers"] = {**kwargs.get("headers", {"Accept": value}), **{"Accept": value}}
            return func(instance, *args, **kwargs)

        return __inner__

    return __wrapper__


def contentType(*, value: str):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象

    位置：针对装饰器所在的函数，发送请求时，临时新增的 Content-Type Header.

    作用: 新增临时 Content-Type Header

    :param value:
    :return:
    """
    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="headers.contentType", expect=BigTangerine)
        def __inner__(instance: BigTangerine, *args, **kwargs):
            kwargs["headers"] = {**kwargs.get("headers", {"Content-Type": value}), **{"Content-Type": value}}
            return func(instance, *args, **kwargs)

        return __inner__

    return __wrapper__


def addHeaders(*, headers: dict):
    """
    使用条件: 作用在 类型 BigTangerine 或 其子类的 对象

    位置：针对装饰器所在的函数，发送请求时，临时新增的Header.

    作用: 新增临时 Header

    :param headers:
    :return:
    """
    def __wrapper__(func):
        @_get_method_params(method=func)
        @_check_instance(decorator="headers.addHeaders", expect=BigTangerine)
        def __inner__(instance: BigTangerine, *args, **kwargs):
            kwargs["headers"] = {**kwargs.get("headers", headers), **headers}
            return func(instance, *args, **kwargs)

        return __inner__

    return __wrapper__



import os
import queue
import threading


# TODO 集成logging模块
def haha(*, LogPath: str, LogName: str, Mode: str = "a"):
    def __wrapper__(func):
        def __inner__(msg: dict):
            logger = smile.log(path=LogPath, file_name=LogName, mode=Mode)
            logger.append_msg(msg=msg)
            logger.printer(func)

        return __inner__

    return __wrapper__


lock = threading.Lock()


class SmileVoice(threading.Thread):
    ThreadName = "Smile Log Thread"

    def __init__(self, path, file_name="orange.log", mode='a'):
        super().__init__(name=SmileVoice.ThreadName)
        self.path = path
        self.queue = queue.Queue()
        self.file_name = file_name
        self.__formatter = None
        self.mode = mode
        self.start()

    def append_msg(self, *, msg: dict) -> None:
        self.queue.put(msg)

    def printer(self, formatter) -> None:
        """
        set how to write the log
        :param formatter:
        :return:
        """
        self.__formatter = formatter

    def run(self) -> None:
        """
        日志线程结束条件：没有输出的消息 及 主线程已结束
        """
        while True:
            if not self.queue.empty():
                msg = self.queue.get()
                with open(os.path.join(self.path, self.file_name), self.mode, encoding="utf-8") as f:
                    self.__formatter(msg, f)
            if self.queue.empty() and not threading.main_thread().is_alive():
                break


class Smile(object):
    """
    control log thread
    one log file , one log thread
    """

    def __init__(self):
        self.loggers = {}

    def log(self, *, path: str, file_name: str, mode: str) -> SmileVoice:
        """
        创建记录日志的线程
        如果记录的日志线程已存在，则不会创建新线程
        :param path: 日志存放路径
        :param file_name: 日志文件名
        :param mode: 写文件的模式
        :return:
        """
        try:
            lock.acquire()
            if self.loggers.get(file_name) is None:
                self.loggers[file_name] = SmileVoice(path=path, file_name=file_name, mode=mode)
            return self.loggers.get(file_name)
        finally:
            lock.release()


# 当引入该package时,日志线程自动启动
smile = Smile()



import re
import warnings


class MethodWrongErr(Exception): ...


class InstanceWrongErr(Exception): ...


class JwtWrongErr(Exception): ...


class RequestParamErr(Exception): ...


class NetWorkErr(Exception): ...


class JsonPathNotFoundErr(Exception): ...


class FactoryParamsWarning(UserWarning): ...


class NoMetaErr(Exception): ...


class UnSupportCharErr(Exception): ...


def _check_method(*, decorator: str, method, expect):
    def __wrapper__(func):
        def __inner__(ins, *args, **kwargs):
            if not method.__name__ == expect: raise MethodWrongErr(f"{decorator} should on the {expect} method")
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _check_instance(*, decorator: str, expect):
    def __wrapper__(func):
        def __inner__(ins, *args, **kwargs):
            if not expect.is_class(ins): raise InstanceWrongErr(f"{decorator} should on the {expect.__name__}")
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _check_uri(*, uri: str):
    def __wrapper__(func):
        def __inner__(ins, *args, **kwargs):
            for variable in re.findall(r'{(\w+)}', uri):
                if not kwargs.get("path_variable"):
                    raise RequestParamErr(f"{uri} contain variable,the params must have 'path_variable'")
                if type(kwargs.get("path_variable")) != dict:
                    raise RequestParamErr("path_variable must is dict")
                if not kwargs.get("path_variable").get(variable):
                    raise RequestParamErr(f"{uri} contain {variable}, but path_variable do not have '{variable}'")
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _check_params_factory(*, clazz):
    """
    检查 调用 Factory装饰器时，函数参数中是否包含 Meta 对象
    如果 传入的参数包含多个参数则提示错误
    :param clazz:  MetaJson 或 MetaMysql
    :return:
    """

    def __wrapper__(func):
        def __inner__(ins, *args, **kwargs):

            params = kwargs.get("method.__annotations__").values()
            # 计算是否包含形如 obj: clazz 这样的参数
            count = len(list(filter(lambda param: issubclass(param, clazz), params)))
            # 计算是否包含形如 obj: list[clazz] 这样的参数
            if count != 1:
                count += len(list(
                    filter(lambda param: param.__origin__ == list and issubclass(param.__args__[0], clazz), params)))
            # 只能包含其中的任一一个
            if count != 1: warnings.warn(f"{clazz} params number should = 1, Now {count}", FactoryParamsWarning)
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _check_meta_exist(*, meta_name: str):
    def __wrapper__(func):
        def __inner__(ins, *args, **kwargs):
            if not kwargs.get(meta_name):
                raise NoMetaErr(f'{meta_name} is not exists')
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__


def _un_support_char(*, content: str, char: str, err_msg: str):
    def __wrapper__(func):
        def __inner__(ins, *args, **kwargs):
            if char in content:
                raise UnSupportCharErr(err_msg)
            return func(ins, *args, **kwargs)

        return __inner__

    return __wrapper__



import sys

from cjen import BigTangerine, JWTFrom, JWTAction
import cjen


class JWTMockService(BigTangerine):
    @cjen.http.base_url(uri="http://127.0.0.1:5000")
    def __init__(self):
        super().__init__()

    @cjen.http.post_mapping(uri="login")
    @cjen.jwt(json_path="$.payload.jwtAuthenticationResponse.token", key="jwt", jwt_from=JWTFrom.BODY,
              action=JWTAction.INIT)
    def login(self, data, resp=None, **kwargs): ...

    @cjen.http.get_mapping(uri="init_in_header")
    @cjen.jwt(json_path="$.token", key="jwt", jwt_from=JWTFrom.HEADER, action=JWTAction.INIT)
    def init_in_header(self, resp=None, **kwargs): ...

    @cjen.http.get_mapping(uri="init_in_body")
    @cjen.jwt(json_path="$.payload.jwtAuthenticationResponse.token", key="jwt", jwt_from=JWTFrom.BODY,
              action=JWTAction.INIT)
    def init_in_body(self, resp=None, **kwargs): ...

    @cjen.http.get_mapping(uri="exchange_in_header")
    @cjen.jwt(json_path="$.token", key="jwt", jwt_from=JWTFrom.HEADER,
              action=JWTAction.EXCHANGE)
    def exchange_in_header(self, resp=None, **kwargs): ...

    @cjen.http.get_mapping(uri="exchange_in_body")
    @cjen.jwt(json_path="$.payload.jwtAuthenticationResponse.token", key="jwt", jwt_from=JWTFrom.BODY,
              action=JWTAction.EXCHANGE)
    def exchange_in_body(self, resp=None, **kwargs): ...

    @cjen.http.get_mapping(uri="refresh_in_header")
    @cjen.jwt(json_path="$.token", key="jwt", jwt_from=JWTFrom.HEADER,
              action=JWTAction.REFRESH)
    def refresh_in_header(self, resp=None, **kwargs): ...

    @cjen.http.get_mapping(uri="refresh_in_body")
    @cjen.jwt(json_path="$.payload.jwtAuthenticationResponse.token", key="jwt", jwt_from=JWTFrom.BODY,
              action=JWTAction.REFRESH)
    def refresh_in_body(self, resp=None, **kwargs): ...


def test_init_body_exchange_body_refresh_body():
    mock = JWTMockService()
    mock.init_in_body()
    assert mock.headers["jwt"] == "Jwt Init Token In Body"
    mock.exchange_in_body()
    assert mock.headers["jwt"] == "Jwt Exchange Token In Body"
    mock.refresh_in_body()
    assert mock.headers["jwt"] == "Jwt Refresh Token In Body"


def test_init_header_exchange_header_refresh_header():
    mock = JWTMockService()
    mock.init_in_header()
    assert mock.headers["jwt"] == "Jwt Init Token In Header"
    mock.exchange_in_header()
    assert mock.headers["jwt"] == "Jwt Exchange Token In Header"
    mock.refresh_in_header()
    assert mock.headers["jwt"] == "Jwt Refresh Token In Header"



from cjen.bigtangerine import ContextManager


class MetaData(object):
    """
    无法配合使用 装饰器 @property
    """
    def __init__(self):
        self.cols = list(filter(
            lambda method: callable(getattr(self, method)) and not method.startswith("_") and not method.endswith(
                "_") and method != "factory" and method != "is_class"
            ,
            dir(self)))
        self.meta_data = dict.fromkeys(self.cols, None)
        self.meta_source = None
        self.context = ContextManager()

    @classmethod
    def is_class(cls, instance):
        return isinstance(instance, MetaData)

    @staticmethod
    def factory(*, clazz, data):
        meta = clazz()
        meta.meta_source = data
        return meta


class MetaJson(MetaData): ...


class MetaMysql(MetaData): ...



import pymysql
from dbutils.pooled_db import PooledDB
from pymysql.cursors import Cursor


class DatabasePool(object):
    __pools = {}

    @classmethod
    def cursor(cls, host: str, user: str, pwd: str, database: str, port: int = 3306) -> Cursor:
        session_name = f'{host}{user}{pwd}{port}{database}'
        if not cls.__pools.get(session_name):
            cls.__pools[session_name] = PooledDB(
                creator=pymysql,
                maxconnections=5,
                mincached=1,
                maxcached=0,
                blocking=True,
                host=host,
                port=port,
                user=user,
                password=pwd,
                database=database,
                charset='utf8mb4',
            )
        return cls.__pools.get(session_name).connection().cursor()
